#include <max6675.h>
#include <TimedAction.h>

// Thermocouple pour les petites céramiques
int thermoDO = 2;
int thermoCS = 3;
int thermoCLK = 4;
int vccPin = 5;
MAX6675 thermocouplePtCer(thermoCLK, thermoCS, thermoDO);

int pinMoteurPWM = 6;

//Potentiometre réglage accélération
// Pin 7 VCC potentiometre
int pinVccPota = 7;
//Pin 8 GND potentiometre
int pinGndPota = 8;
//Pin valeur potentiometre
int pinValPota = A5;

// Thermocouple pour les grandes céramiques
int thermoDO1 = 9;
int thermoCS1 = 10;
int thermoCLK1 = 11;
int vccPin1 = 12;
int gndPin1 = 13;
MAX6675 thermocoupleGdCer(thermoCLK1, thermoCS1, thermoDO1);

// Pin de control des relais
int pinMoteur = A0; // Pin de controle du relai moteur
int pinVentilHigh = A1; // Pin de controle du relai ventilateur/lumière
int pinGdCer =A2; //Pin de controle du relai grande céramique
int pinPtCer = A3; //Pin de controle du relai petite céramique

// Les controles logique du relai fait que si on mets une pin sur Low on active le relai (et donc laisse passer le courant)
// Donc on inverse virtuellement la logique pour la compréhension du code
boolean on = LOW;
boolean off = HIGH;

// On initialise les valeurs qui recevront la température des thermocouple
int realTempGdCer = 0; //Température grande céramique
int realTempPtCer = 0; //Température petite céramique

// On initialise les valeurs cible de température
//Valeur theorique previsionnel
//int targetTempGdCer = 260; 
//int targetTempPtCer = 350;
//Valeur definit apres test
int targetTempGdCer = 280;
int targetTempPtCer = 320;

// La valeur renvoyé par les thermocouple n'est pas forcément la valeur réel, il faut un coefficien multiplicateur (a tester et définir)
float coeffGdCer = 0.60;
float coeffPtCer = 0.55;
//Innitialisation de variables diverses
boolean moteurOn = false;
int timing = 0;

//Variable utilitaire
// Declaration des Timedaction(toutes les x millisecondes, function)
TimedAction timedActionChauffe = TimedAction(1000,chauffe);
TimedAction timedActionLecture = TimedAction(2000,lecture);
TimedAction timedActionVentilation = TimedAction(10000,ventilation);
TimedAction timedActionMotorisationOn = TimedAction(10000,motorisationOn);
TimedAction timedActionMotorisationOff = TimedAction(10000,motorisationOff);


void setup(){
  Serial.begin(9600);
  pinMode(vccPin, OUTPUT); digitalWrite(vccPin, off);
  pinMode(vccPin1, OUTPUT); digitalWrite(vccPin1, off);
  pinMode(gndPin1, OUTPUT); digitalWrite(gndPin1, on);
  pinMode(pinMoteur, OUTPUT); digitalWrite(pinMoteur, off);
  pinMode(pinVentilHigh, OUTPUT); digitalWrite(pinVentilHigh, off);
  pinMode(pinGdCer, OUTPUT); digitalWrite(pinGdCer, off);
  pinMode(pinPtCer, OUTPUT); digitalWrite(pinPtCer, off);
  pinMode(pinVccPota, OUTPUT); digitalWrite(pinVccPota, HIGH);
  pinMode(pinGndPota, OUTPUT); digitalWrite(pinGndPota, LOW);
  pinMode(pinValPota,INPUT);
  // Par sécurité on fait une pause de 1s pour la fin du setup
  delay(1000);
}
//Fonctionnement
void loop() {
  if (realTempPtCer >= targetTempPtCer) { // Premiere fois qu'on atteint la température cible
    motorisationOn();
  }
  // Toutes les secondes on regarde la température et on active ou non les relais de chauffe
  timedActionChauffe.check();
  // Toutes les 2 secondes on vérifie notre température et on met à jour la donnée
  timedActionLecture.check();
  // Toutes les 10 secondes on regarde la température et on active ou non la ventilation haute et la lumière
  timedActionVentilation.check();
  // Toutes les 500ms on active ou non le moteur
  Serial.println("ValPOta");
  Serial.println(analogRead(pinValPota));
  Serial.println("VAlPOtaEnd");
  delay(500);
}
//Lecture de la temperature
void lecture() {
  realTempPtCer = thermReading(thermocouplePtCer,coeffPtCer);
  Serial.println("TempPtCer");
  Serial.println(realTempPtCer);
  realTempGdCer = thermReading(thermocoupleGdCer, coeffGdCer);
  Serial.println("TempGdCer");
  Serial.println(realTempGdCer);
}
//Mise en chauffe ou non des ceramiques
void chauffe(){
  if (realTempPtCer <= targetTempPtCer) {
    digitalWrite(pinPtCer, on);
  } else {
    digitalWrite(pinPtCer, off);
  }
  if (realTempGdCer <= targetTempGdCer) {
    digitalWrite(pinGdCer, on);
  } else {
    digitalWrite(pinGdCer, off);
  }
}
//Activation de la ventilation si on atteint la moitie de la valeur cible
void ventilation() {
    if (realTempPtCer >= targetTempPtCer/2) {
    digitalWrite(pinVentilHigh, on);
  } else {
    digitalWrite(pinVentilHigh, off);
  }
}
//Demarrage de l'avancement du tapis
void motorisationOn() {
  int valPota = analogRead(pinValPota);
  int fadMax = 0;
  Serial.println("Motor On");
  digitalWrite(pinMoteur, on);
  if (valPota <= 500) {
    fadMax = 0;
  } else {
    fadMax = valPota/4;
    if(fadMax > 255) {
      fadMax = 255; 
    }
  }
  
    analogWrite(pinMoteurPWM, fadMax);         
    // wait for 30 milliseconds to see the dimming effect    

 Serial.println("Motor On speed of work done"); 
}
//Arret de l'avancement du tapis
void motorisationOff() {
  Serial.println("Motor Off'");
  digitalWrite(pinMoteur, off);
  analogWrite(pinMoteurPWM, 0);
}

// Fonction de lecture des thermocouple, on lit 5 fois et on retourne la moyenne
int thermReading(MAX6675 therm, float coeff) {
  int delai = 10; //Delai en ms entre chacune des nb_interro lecture
  int nb_interro = 5; // Nb de lecture
  int result = 0; // Stockage de la somme des résultats
   for (int compteur=1; compteur <= nb_interro; compteur++) {
      result = result + therm.readCelsius();
    delay (delai);
   }
  return (result/nb_interro) * coeff;
}
